@@ -223,6 +223,20 @@ class BcbioLSFControllerLauncher(launcher.LSFControllerLauncher):
         return super(BcbioLSFControllerLauncher, self).start()
 
 # ## Sun Grid Engine (SGE)
+
+def _local_environment_exports():
+    """Create export string for a batch script with environmental variables to pass.
+
+    Passes additional environmental variables not inherited by some schedulers.
+    LD_LIBRARY_PATH filtered by '-V' on recent Grid Engine releases.
+    """
+    exports = []
+    for envname in ["LD_LIBRARY_PATH"]:
+        envval = os.environ.get(envname)
+        if envval:
+            exports.append("export %s=%s" % (envname, envval))
+    return "\n".join(exports)
+
 class BcbioSGEEngineSetLauncher(launcher.SGEEngineSetLauncher):
     """Custom launcher handling heterogeneous clusters on SGE.
     """
@@ -231,18 +245,21 @@ class BcbioSGEEngineSetLauncher(launcher.SGEEngineSetLauncher):
     pename = traitlets.Unicode("", config=True)
     resources = traitlets.Unicode("", config=True)
     mem = traitlets.Unicode("", config=True)
+    memtype = traitlets.Unicode("mem_free", config=True)
     tag = traitlets.Unicode("", config=True)
+    queue_template = Unicode('')
     default_template = traitlets.Unicode("""#$ -V
 #$ -cwd
-#$ -b y
+#$ -w w
 #$ -j y
 #$ -S /bin/sh
-#$ -q {queue}
 #$ -N {tag}-e
 #$ -t 1-{n}
 #$ -pe {pename} {cores}
+{queue}
 {mem}
 {resources}
+{exports}
 echo \($SGE_TASK_ID - 1\) \* 0.5 | bc | xargs sleep
 %s %s --profile-dir="{profile_dir}" --cluster-id="{cluster_id}"
 """ % (' '.join(map(pipes.quote, engine_cmd_argv)),
@@ -251,35 +268,62 @@ echo \($SGE_TASK_ID - 1\) \* 0.5 | bc | xargs sleep
     def start(self, n):
         self.context["cores"] = self.cores
         if self.mem:
-            self.context["mem"] = "#$ -l mem_free=%sM" % int(float(self.mem) * 1024)
+            if self.memtype == "rss":
+                self.context["mem"] = "#$ -l rss=%sM" % int(float(self.mem) * 1024 / self.cores)
+            else:
+                self.context["mem"] = "#$ -l mem_free=%sM" % int(float(self.mem) * 1024)
         else:
             self.context["mem"] = ""
+        if self.queue:
+            self.context["queue"] = "#$ -q %s" % self.queue
+        else:
+            self.context["queue"] = ""
         self.context["tag"] = self.tag if self.tag else "bcbio"
         self.context["pename"] = str(self.pename)
-        self.context["resources"] = "\n".join(["#$ -l %s" % r.strip()
+        self.context["resources"] = "\n".join([_prep_sge_resource(r)
                                                for r in str(self.resources).split(";")
                                                if r.strip()])
+        self.context["exports"] = _local_environment_exports()
         return super(BcbioSGEEngineSetLauncher, self).start(n)
 
 class BcbioSGEControllerLauncher(launcher.SGEControllerLauncher):
     batch_file_name = Unicode(unicode("sge_controller" + str(uuid.uuid4())))
     tag = traitlets.Unicode("", config=True)
     resources = traitlets.Unicode("", config=True)
+    queue_template = Unicode('')
     default_template = traitlets.Unicode(u"""#$ -V
-#$ -S /bin/sh
 #$ -cwd
+#$ -w w
+#$ -S /bin/sh
 #$ -N {tag}-c
+{queue}
 {resources}
+{exports}
 %s --ip=* --log-to-file --profile-dir="{profile_dir}" --cluster-id="{cluster_id}" %s
 """ % (' '.join(map(pipes.quote, controller_cmd_argv)),
        ' '.join(controller_params)))
     def start(self):
         self.context["tag"] = self.tag if self.tag else "bcbio"
-        self.context["resources"] = "\n".join(["#$ -l %s" % r.strip()
+        self.context["resources"] = "\n".join([_prep_sge_resource(r)
                                                for r in str(self.resources).split(";")
                                                if r.strip()])
+        if self.queue:
+            self.context["queue"] = "#$ -q %s" % self.queue
+        else:
+            self.context["queue"] = ""
+        self.context["exports"] = _local_environment_exports()
         return super(BcbioSGEControllerLauncher, self).start()
 
+def _prep_sge_resource(resource):
+    """Prepare SGE resource specifications from the command line handling special cases.
+    """
+    resource = resource.strip()
+    k, v = resource.split("=")
+    if k in set(["ar"]):
+        return "#$ -%s %s" % (k, v)
+    else:
+        return "#$ -l %s" % resource
+
 def _find_parallel_environment(queue):
     """Find an SGE/OGE parallel environment for running multicore jobs in specified queue.
     """
@@ -703,23 +747,26 @@ def _scheduler_resources(scheduler, params, queue):
     but are specific to different environments.
     """
     resources = copy.deepcopy(params.get("resources", []))
+    specials = {}
     if not resources:
         resources = []
     if isinstance(resources, basestring):
         resources = resources.split(";")
-    pename = None
     if scheduler in ["SGE"]:
         pass_resources = []
         for r in resources:
             if r.startswith("pename="):
                 _, pename = r.split("=")
+                specials["pename"] = pename
+            elif r.startswith("memtype="):
+                _, memtype = r.split("=")
+                specials["memtype"] = memtype
             else:
                 pass_resources.append(r)
-        if pename is None:
-            pename = _find_parallel_environment(queue)
         resources = pass_resources
-
-    return ";".join(resources), pename
+        if "pename" not in specials:
+            specials["pename"] = _find_parallel_environment(queue)
+    return ";".join(resources), specials
 
 def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
            extra_params):
@@ -738,7 +785,7 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
                "your scheduler. If it should, please file a bug report at "
                "http://github.com/roryk/ipython-cluster-helper. Thanks!")
         sys.exit(1)
-    resources, pename = _scheduler_resources(scheduler, extra_params, queue)
+    resources, specials = _scheduler_resources(scheduler, extra_params, queue)
     if scheduler in ["OLDSLURM", "SLURM"]:
         resources, slurm_atrs = get_slurm_attributes(queue, resources)
     else:
@@ -765,8 +812,10 @@ def _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id,
          "--cluster-id=%s" % (cluster_id)
          ]
     args += _get_profile_args(profile)
-    if pename:
-        args += ["--%s.pename=%s" % (engine_class, pename)]
+    if specials.get("pename"):
+        args += ["--%s.pename=%s" % (engine_class, specials["pename"])]
+    if specials.get("memtype"):
+        args += ["--%s.memtype=%s" % (engine_class, specials["memtype"])]
     if slurm_atrs:
         args += ["--%s.machines=%s" % (engine_class, slurm_atrs.get("machines", "0"))]
         args += ["--%s.account=%s" % (engine_class, slurm_atrs["account"])]
@@ -796,18 +845,6 @@ def _stop(profile, cluster_id):
     args += _get_profile_args(profile)
     subprocess.check_call(args)
 
-def _is_up(url_file, n):
-    try:
-        client = Client(url_file, timeout=60)
-        up = len(client.ids)
-        client.close()
-    except iperror.TimeoutError:
-        return False
-    except IOError:
-        return False
-    else:
-        return up >= n
-
 @contextlib.contextmanager
 def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
                  start_wait=16, extra_params=None, retries=None, direct=False):
@@ -820,6 +857,9 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
         Defaults to 16 minutes. Set to longer for slow starting clusters.
       - retries: Number of retries to allow for failed tasks.
     """
+    num_jobs = int(num_jobs)
+    cores_per_job = int(cores_per_job)
+    start_wait = int(start_wait)
 
     if extra_params is None:
         extra_params = {}
@@ -844,7 +884,7 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
     while 1:
         try:
             if extra_params.get("run_local"):
-                _start_local(cores_per_job, profile, cluster_id)
+                _start_local(num_jobs, profile, cluster_id)
             else:
                 _start(scheduler, profile, queue, num_jobs, cores_per_job, cluster_id, extra_params)
             break
@@ -885,23 +925,20 @@ def cluster_view(scheduler, queue, num_jobs, cores_per_job=1, profile=None,
 
 def _nengines_up(url_file):
     "return the number of engines up"
+    client = None
     try:
         client = Client(url_file, timeout=60)
         up = len(client.ids)
+        client.close()
     # the controller isn't up yet
     except iperror.TimeoutError:
-        return False
-        client.close()
         return 0
     # the JSON file is not available to parse
     except IOError:
-        return False
-        client.close()
         return 0
     else:
         return up
 
-
 def _get_balanced_blocked_view(client, retries):
     view = client.load_balanced_view()
     view.set_flags(block=True)
@@ -911,8 +948,6 @@ def _get_balanced_blocked_view(client, retries):
 
 def _shutdown(client):
     print "Sending a shutdown signal to the controller and engines."
-#    client.spin()
-#    client.shutdown(hub=True, block=False)
     client.close()
 
 def _get_direct_view(client, retries):
